<?php
namespace DiviBooster\PhpCode;

class BuilderContent {
	
	protected $builderContent;
	
	static function fromJson($json) {
		return new self(json_decode($json, true));
	}
	
	static function fromAssoc($assoc) {
		return new self($assoc);
	}
	
	function __construct($shortcodeObject) {
		$this->builderContent = $shortcodeObject;
	}
	
	function mapModuleAttributes($callback, $moduleType=null) {
		$sections = $this->builderContent; 
		if (is_array($sections)) {
			foreach($sections as $key=>$section) {
				$sections[$key] = $this->mapChildren($section, $callback, $moduleType);
			}
		}
		$this->builderContent = $sections;
		return $this;
	}

	protected function mapChildren($element, $callback, $moduleType) {
		if ($this->isContainerElement($element)) {
			foreach($element['content'] as $key=>$child) {
				if ($this->isContainerElement($child)) {
					$element['content'][$key] = $this->mapChildren($child, $callback, $moduleType);
				} else {
					if ($moduleType && !(isset($child['type']) && $child['type'] === $moduleType)) {
						continue; 
					}
					if (!isset($child['attrs']) || !is_array($child['attrs'])) {
						continue;
					}
					$element['content'][$key]['attrs'] = call_user_func($callback, $child['attrs']);
				}
			}
		}
		return $element;
	}

	protected function isContainerElement($element) {
		return (isset($element['content']) && is_array($element['content']));
	}
	
	function toJson() {
		return json_encode($this->builderContent);
	}
	
	function toAssoc() {
		return $this->builderContent;
	}
}