<?php
namespace DiviBooster\PhpCode;

include_once(dirname(__FILE__).'/filter-et_pb_db_php_code_pre_save.php');

// === Load features ===

include_once(dirname(__FILE__).'/features/role_editor/role_editor.php');


// === Load jsx ===

if (function_exists('add_action')) {
    add_action('et_fb_enqueue_assets', __NAMESPACE__.'\\enqueue_vb_scripts');
    add_action('admin_enqueue_scripts', __NAMESPACE__.'\\enqueue_vb_scripts');
}
function enqueue_vb_scripts() {
	wp_enqueue_script("dbphp_db_php_code-builder-bundle", plugin_dir_url(__FILE__)."/visual-builder.js", array('react', 'react-dom'), DBMPHP_VERSION);
}

add_action('et_builder_ready', __NAMESPACE__.'\\load_php_code_module');

function load_php_code_module() {

    if (!class_exists('\ET_Builder_Module')) { return; }

    class PhpCodeModule extends \ET_Builder_Module {
	
        public $slug = 'et_pb_db_php_code';
        public $vb_support = 'on';
    
        protected $module_credits = array(
            'module_uri' => 'https://divibooster.com/',
            'author' => 'Divi Booster',
            'author_uri' => 'https://divibooster.com'
        );
    
        function __construct() {
            parent::__construct();
        }
    
        function init() {
    
            $this->name = esc_html__('PHP Code', 'divi-module-php-code');
    
            // wptexturize is often incorrectly parsed single and double quotes
            // This disables wptexturize on this module
            add_filter('no_texturize_shortcodes', array($this, 'disable_wptexturize'));	
    
            do_action('dbphp_et_pb_db_php_code_init', $this);
        }
        
        function get_advanced_fields_config() {
            return array(
                'background' => array(),
                'custom_margin_padding' => array(
                    'css' => array(
                        'important' => array( 'custom_margin' ), // needed to overwrite last module margin-bottom styling
                    ),
                ),
                'max_width' => array(),
                'text'      => false,

                // Add font styles for title and body text
                'fonts' => array(
                    'title' => array(
                        'label'    => esc_html__('Title', 'divi-module-php-code'),
                        'css'      => array(
                            'main' => "%%order_class%% .dbphp_title",
                            'important' => 'all',
                        ),
                    ),
                    'code_output' => array(
                        'label'    => esc_html__('Code Output', 'divi-module-php-code'),
                        'css'      => array(
                            'main' => "%%order_class%% .dbphp_output",
                            'important' => 'all',
                        ),
                    ),
                ),

                'filters' => array(),
            );
        }
        
        function get_settings_modal_toggles() {
            return array(
                'general'  => array(
                    'toggles' => array(
                        'et_pb_db_php_code_main_content' => array(
                            'title' => esc_html__('Code', 'divi-module-php-code'),
                            'always_open' => true,
                        ),
                    ),
                )
            );
        }
    
        function get_fields() {
            $fields = array(
                'title' => array(
                    'label'           => esc_html__( 'Title', 'et_builder' ),
                    'type'            => 'text',
                    'option_category' => 'basic_option',
                    'description'     => esc_html__( 'Input a title for the module.', 'et_builder' ),
                    'toggle_slug'     => 'et_pb_db_php_code_main_content',
                ),
                'code' => array(
                    'label' => 'PHP Code',
                    'type' => 'codemirror',
                    'mode' => 'html',
                    'option_category' => 'basic_option',
                    'description' => 'Enter the PHP Code to be executed by the module.',
                    'toggle_slug' => 'et_pb_db_php_code_main_content',
                ), 
                'code_to_render' => array(
                    'label' => 'PHP Code to Render',
                    'type' => 'codemirror',
                    'mode' => 'html',
                    'option_category' => 'basic_option',
                    'description' => 'Main code will be copied here to trigger computed callback.',
                    'toggle_slug' => 'et_pb_db_php_code_main_content',
                ), 
                'code_base64' => array(
                    'label' => 'PHP Code (Base64)',
                    'type' => 'hidden',
                    'option_category' => 'basic_option',
                    'description' => 'Base64 encoded snapshot of code.',
                    'toggle_slug' => 'et_pb_db_php_code_main_content',
                ), 
                'code_hash' => array(
                    'label' => 'PHP Code Hash',
                    'type' => 'hidden',
                    'option_category' => 'basic_option',
                    'description' => 'Hash of snapshot of code.',
                    'toggle_slug' => 'et_pb_db_php_code_main_content',
                ), 
                'debug' => array(
                    'label' => 'Enable Debugging',
                    'type' => 'yes_no_button',
                    'option_category' => 'configuration',
                    'options' => array(
                        'off' => esc_html__('Off', 'et_builder'), 
                        'on'  => esc_html__('On', 'et_builder'), 
                    ),
                    'default' => 'off',
                    'description' => 'When enabled, errors in the PHP code will be displayed on the front end.',
                    'toggle_slug' => 'et_pb_db_php_code_main_content'
                ),
                '__shortcode_output' => array(
                    'type' => 'computed',
                    'computed_callback' => __NAMESPACE__.'\\execute_code',
                    'computed_depends_on' => apply_filters(
                        'dbphp_db_php_code_computed_depends_on', 
                        array(
                            'code_to_render',
                            'debug',
                            'code_base64',
                            'code_hash',
                        )
                    )
                )
            );
            return apply_filters('dbphp_et_pb_db_php_code_get_fields', $fields, $this);
        }
        
        public function render($unprocessed_props, $content, $render_slug) {
            $props = $this->props;
            $title = $props['title'];
            $title_html = (!empty($props['title']) ? '<h3 class="dbphp_title">' . esc_html( $title ) . '</h3>' : '' );
            $code_output = execute_code($props);
            $rendered = sprintf(
                '%1$s
                <div class="dbphp_output entry-content">
                    %2$s
                </div>',
                $title_html,
                $code_output
            );

            // ET converts 'target' attrib to 'data-et-target-link' to avoid WP processing. Convert it back now.
            $rendered = str_replace(' data-et-target-link=', ' target=', $rendered);

            return $rendered;
        }
    }
	new PhpCodeModule;
}

// === Computed callback handler ===

add_action('wp_ajax_et_pb_process_computed_property', __NAMESPACE__.'\\add_hash_to_computed_callback', 9);

function add_hash_to_computed_callback() {
    	
	// Ensure user allowed to edit php code module in role editor
	if (!(function_exists('et_pb_is_allowed') && et_pb_is_allowed('et_pb_db_php_code'))) { 
		return; 
	}

    if (!isset($_POST['module_type']) || $_POST['module_type'] !== 'et_pb_db_php_code') {
        return;
    }
    if (isset($_POST['depends_on']['code_to_render'])) {
        $code_output = stripslashes($_POST['depends_on']['code_to_render']);

        // Remove first line (nonce) from $code_output
        $code_output = preg_replace('/^.*\n/', '', $code_output);

        $_POST['depends_on']['code_base64'] = base64_encode($code_output);
        $_POST['depends_on']['code_hash'] = dbmphp_get_hash($_POST['depends_on']['code_base64']);
    }
}

// === Code execution ===

function execute_code($props) {

    $props = wp_parse_args(
        $props, 
        array(
            //'code' => '',
            'code_base64' => '',
            'code_hash' => '',
        )
    );

    // Check code integrity
    $verified = is_valid_hash($props['code_hash'], $props['code_base64']);
    if (!$verified) {
        return error('error', __('PHP code integrity check failed. Please check the module settings and resave.', 'divi-module-php-code'), $props);
    }
    
    // Retrieve the code
    $code = base64_decode($props['code_base64']);

            // // Remove bfb html encoding
            // $code = html_entity_decode($code);

    // Remove opening php tags and any whitespace before and after them
    $code = preg_replace('/^\s*<\?php\s*/s', '', $code);

    // Execute the code
    try {
        $debug_enabled = (isset($props['debug']) && $props['debug']==='on');
        $output = (PhpCode::fromString($code))->output($debug_enabled);
    } catch (\ParseError $e) {
        return error('parse_error', $e->getMessage(), $props);
    } catch (\Exception $e) {
        return error('error', $e->getMessage(), $props);
    }

    // Return the result
    return $output;
}

function error($type, $msg, $props) {
    $debug_enabled = (isset($props['debug']) && $props['debug']==='on');
    if (!$debug_enabled) {
        $type = 'error';
        $msg = __('Content could not be displayed', 'divi-module-php-code');
    }
    $error_types = array(
        'parse_error' => __('Parse Error', 'divi-module-php-code'),
        'error' => __('Error', 'divi-module-php-code')
    );
    return sprintf(
        /* translators: 1: Error type 2: Error message */
        __('%1$s: %2$s.', 'divi-module-php-code'),
        $error_types[$type],
        esc_html($msg)
    );
}


// === Base64 encode the code on save ===

add_filter('et_pb_db_php_code_pre_save', __NAMESPACE__.'\\dbphp_base64_on_save');

function dbphp_base64_on_save($shortcode) {
	
	// Ensure user allowed to edit php code module in role editor
	if (!(function_exists('et_pb_is_allowed') && et_pb_is_allowed('et_pb_db_php_code'))) { 
		return $shortcode; 
	}
	
	if (($shortcode instanceof \DiviBooster\PhpCode\Shortcode)) { 
		$code = stripslashes($shortcode->code);

        
        $code = str_replace('+', '%2B', $code); // Preserve + signs

        // Decode square brackets
        $code = str_replace('%91', '[', $code);
        $code = str_replace('%93', ']', $code);

        $code = urldecode($code);

        // Remove html encoding on bfb code
        $is_bfb = (!empty($_POST['options']['conditional_tags']['is_bfb']));
        $is_edit_post = (!empty($_POST['action']) && $_POST['action']==='editpost' && !empty($_POST['et_pb_use_builder']) && $_POST['et_pb_use_builder']==='on');
        if ($is_bfb || $is_edit_post) {
            $code = html_entity_decode($code);
        }

		$encoded_content = base64_encode($code); // Base64 to better avoid formatting issues
        $shortcode = $shortcode->withAttribute('code_base64', $encoded_content);
		$shortcode = $shortcode->withAttribute('code_hash', dbmphp_get_hash($encoded_content));
	}
	return $shortcode;
}


// === Hash functions ===

function is_valid_hash($hash, $text) {
    return (!empty($hash) && dbmphp_get_hash($text) === $hash);
}

function dbmphp_get_hash($content) {
	$key = get_option('dbmphp_hmac_key', '');
    if (!empty($key) && function_exists('hash_hmac')) {
	    return hash_hmac('sha256', trim($content), $key);
    } 
    return false;
}

// === Generate key if not set ===

function dbmphp_generate_hmac_key() {
	if (!get_option('dbmphp_hmac_key')) {
		update_option('dbmphp_hmac_key', wp_generate_password(64, true, true));
	}
}
add_action('init', __NAMESPACE__.'\\dbmphp_generate_hmac_key');


// === Add play button to visual builder ===

add_action('admin_footer', __NAMESPACE__.'\\add_play_button');
add_action('wp_footer', __NAMESPACE__.'\\add_play_button');

function add_play_button() {
?>
<style>
.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] .et-code-snippets-library-btns-wrap:before {
    content: "\45";
    font-size: 27px;
    color:#ccffcc;
    font-family:ETModules;
    cursor: pointer;
    width: 28px;
    height: 27px;
    border-radius: 3px;
    padding-top:1px;
    background-color: rgb(255,255,255, 0.2) !important;
}
.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] .et-fb-form__group:nth-of-type(4) {
    display:none;
}
</style>
<script>
jQuery(document).ready(function($) {
  $(document).on('click', '.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] .et-code-snippets-library-btns-wrap', function(event) {
    if (event.target !== event.currentTarget) {
        return; // Exit the function if the click is on a child element
    }
    var firstEditor = $('.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] .CodeMirror').get(0)?.CodeMirror;
    if (!firstEditor) { return; }
    $('.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] textarea[name="code_to_render"]').click(); // Enable hidden codemirror
    var secondEditor = $('.et-fb-form__toggle[data-name="et_pb_db_php_code_main_content"] .CodeMirror').get(1)?.CodeMirror;
    if (!secondEditor) { return; }
    var content = firstEditor.getValue();

    // Add a random nonce as the first line of content to prevent caching
    var nonce = Math.random().toString(36).substring(2, 15);
    content = '/* nonce:' + nonce + ' */\n' + content;

    secondEditor.setValue(content);
  });
});
</script>
<?php
}

// === Overwrite the unprotected code variable with the verified code on load ===
// * Need to do, otherwise if code was modified in the database, the visual builder will load the modified code and an admin could unknowingly save the modified code

add_filter('et_pb_module_shortcode_attributes', __NAMESPACE__.'\\restore_verified_code', 10, 3);

function restore_verified_code($props, $attrs, $render_slug) {
    if ($render_slug == 'et_pb_db_php_code' && is_array($props)) {
        $props['code'] = '';
        $props['code_to_render'] = '';
        if (isset($props['code_base64']) && isset($props['code_hash']) && is_valid_hash($props['code_hash'], $props['code_base64'])) {
            $props['code'] = base64_decode($props['code_base64']);

            // First line of render code usually has nonce and gets stripped out - add a dummy nonce line here
            $props['code_to_render'] = "/* nonce:ignored */\n".$props['code'];
        }
    }
    return $props;
}